/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/** Content widgets based on a text block. */
module textContent;

import mde.gui.widget.Widget;
import mde.gui.widget.TextWidget;
import mde.gui.exception;
import mde.gui.renderer.IRenderer;

import mde.content.AStringContent;

debug {
    import tango.util.log.Log : Log, Logger;
    private Logger logger;
    static this () {
	logger = Log.getLogger ("mde.gui.widget.textContent");
    }
}

/// Just displays the content. Generic − any IContent.
class DisplayContentWidget : ATextWidget
{
    this (IWidgetManager mgr, widgetID id, WidgetData data, IContent c) {
	content = c;
	WDCMinCheck(data, 1,0, content);
	adapter = mgr.renderer.getAdapter ();
	adapter.text = content.toString(0);
	super (mgr, id, data);
    }
    
protected:
    IContent content;
}

/// Text-box for editing a content. Generic − any AStringContent.
class AStringContentWidget : ATextWidget
{
    this (IWidgetManager mgr, widgetID id, WidgetData data, IContent c) {
	content = cast(AStringContent) c;
	WDCMinCheck(data, 1,0, content);
        adapter = mgr.renderer.getAdapter ();
	adapter.text = content.toString(0);
	super (mgr, id, data);
    }
    
    override bool isWSizable () {    return true;    }
    override bool isHSizable () {    return true;    }
    
    /** On click, request keyboard input. */
    override int clickEvent (wdabs, wdabs, ubyte, bool state) {
	adapter.index = content.editIndex;
	mgr.requestRedraw;
	return 1;	// get keyboard input via keyEvent
    }
    
    override void keyEvent (ushort s, char[] i) {
	adapter.text = content.keyStroke (s, i);
	adapter.index = content.editIndex;
	adapter.getDimensions (mw, mh);	// NOTE: only passively change size: next resize will see new minimal size
	mgr.requestRedraw;
    }
    override void keyFocusLost () {
	adapter.text = content.endEdit;	// update other users of content relying on callbacks
	adapter.index;
	mgr.requestRedraw;
    }
    
protected:
    AStringContent content;
}
